VERSION 8000; ; 8413
* -----------------------------------------------------------------------------
* @filedoc
* @file Topo_TextAusLinienelementen.CSX
* @brief Topografietexte fr Linienlngen und -neigungen generieren
* 
* Dieses Beispielskript generiert Topografietexte fr die Elementlngen und -neigungen vorhandener Topografielinien einer Schicht.
* Die Texte werden in Elementmitte positioniert (Textschicht = Linienschicht).
* Smtliche Parameter werden im Dialog erfasst.
* -----------------------------------------------------------------------------
* Copyright (C) IB&T
*   
* Gewhrleistung und Nutzung
* 
* Die Nutzung des Skriptes erfolgt stets auf eigene Gefahr. IB&T bernimmt keine
* Gewhr dafr, dass die Nutzung dieses Skriptes zu Ihrem gewnschten Ergebnis
* fhrt. Da CardScripte vorhandene Projektdaten durch Direktzugriff auf die
* jeweiligen Datenbanken auch ohne weitere Sicherheitsabfragen unbemerkt verndern
* oder lschen knnen, erfordert eine Nutzung entsprechende Kenntnisse und groe
* Sorgfalt. IB&T haftet nicht fr eventuell entstehende Datenverluste und
* Folgeschden, die durch die Nutzung von CardScripten entstehen.
*
* -----------------------------------------------------------------------------
* @author JH/TK
* @endfiledoc
* -----------------------------------------------------------------------------
*    8300 20.05.09 JH erstellt
*    8312 20.09.10 CK Einfgedatei gendert: Maskfeld.QPR -> Maskfeld_F2.QPC, CONSTTOPOP.QPR -> CONSTTOPO.QPC
*    8402 17.03.11 JH Kodevorbelegung auf 0 gendert  
*    8411 09.08.12 JH auf neue Dialoge umgestellt 
*    8413 01.02.13 CK auf neue Hilfe umgestellt
* @history
* -----------------------------------------------------------------------------
$PRFSTUFE 7

* --- Vereinbarungen ----------------------------------------------------------

QPREINFUEGE "<CARD>CONSTTOPO.QPC"                  | fgt Skript ein: Konstanten fr Textparameter

BILDSCHIRM GRAFIK                                  | Meldungen im Arbeitsprotokoll ausgeben

HILFE SYMBOL "TextausLin";SKRIPTE_TOOLS_TOPOGRAFIE
            
* --- Explizite Deklarationen -------------------------------------------------

SYMBOLE * !Schicht                                 | auszuwertender Schichtname
SYMBOLE * Station; LastStat; DiffStat; TStat       | Linienstation fr Textposition 

SYMBOLE /TOPOSYS/ TopoSys                          | Symbol fr das System der Topografiedaten
SYMBOLE /TOPOLAYER/ Schicht                        | Symbol fr die Schicht
SYMBOLE /TOPOLINE/ Linie                           | Symbol fr die Topografielinie
SYMBOLE /TOPOLINEINDEX/ XLinie                     | Symbol fr das Linienindexobjekt


SYMBOLE * LinKode;LinArt;NiP;!LE;TKode; NKS;THoehe;TAbst
SYMBOLE * RET;LinNr
* --- Implizite Deklarationen -------------------------------------------------
              
LinKode = 0                                        | Linienkode (= Auswahl im Dialog) 
LinArt = 0                                         | Auswertungsart: 0 = Horizontalstrecke, 1 = Raumstrecke

* Textgenerierung
NKS = 2                                            | Anzahl Nachkommastellen 
!LE = " m"                                         | Lngeneinheit 
NIP = 0                                            | Neigungsbeschriftung: 0 = nein, 1 = in Prozent, 2 = in Promille (zurzeit nicht implementiert)
TKode = 0                                          | Textkode (= Auswahl im Dialog)
THoehe = 0,13                                      | Texthhe
TAbst = 0,00                                       | Textabstand zur Linie

* --- Ausfhrung --------------------------------------------------------------

*** Dialogfhrung
Ret = Dialog()
WENN RET == 0 DANN STOP

* Systemobjekt
TopoSys = Project.GetTopoSys()                   | liefert das Objekt fr das System der Topografiedaten

* Schichtobjekt
Schicht = $NULL                                  | setzt das Schichtobjekt auf Null 
Schicht = TopoSys.GetLayerByName(!Schicht)       | liefert das Schichtobjekt 
WENN Schicht == $NULL DANN STOP MELDE "Schicht '#' ist nicht vorhanden --> Textgenerierung wird abgebrochen!"; !Schicht

* Linienindexobjekt: Auswahl der Linien ber Schicht und Kode
XLinie = TopoSys.GetLineIndex(!Schicht)          | liefert Linienindexobjekt der Schicht
XLinie.SetCodeFilter(LinKode)                    | setzt einen Filter fr den Kode

*** Schleife zum Auslesen der Linien ber gesetzte Filter 

* Linienobjekt
SOLANGE Linie = XLinie.GetNext()                 | liest Linien ber gesetzte Filter aus

  LinienText(Linie)                                     | ruft Funktionsblock auf: Elementlngen ermitteln und Text generieren
ENDE SOLANGE

STOP



* --- Funktionsblock --------------------------------------------------------------------------

* -----------------------------------------------------------------------------
* @function DIALOG
* @brief Eingabedialog
*
* @ingroup
* @author JH
* @endfunction
* -----------------------------------------------------------------------------
FUNKTION DIALOG:
  * Deklarationen
  SYMBOLE /DLG_WINDOW/ Dialog
  
  SYMBOLE /DLG_LAYER/ Schicht       | Schicht 
  SYMBOLE /DLG_CODELINE/ Linienkode | Linienkode
    
  SYMBOLE /DLG_Checkbox/ CheckS     | Horizontalstrecke (0) oder Raumstrecke (1)
  SYMBOLE /DLG_Checkbox/ CheckN     | Neigungsbeschriftung ohne (0) in % (1)            
  SYMBOLE /DLG_TEXT/ Einheit        | Lngeneinheit                             
  SYMBOLE /DLG_CODE/  TxCode        | Textkode                                               
  SYMBOLE /DLG_COMBOBOX/ Komma      | Nachkommastellen (0...3)                  
  SYMBOLE /DLG_NUMBER/  TH          | Texthhe (0,13...1,0)                     
  SYMBOLE /DLG_NUMBER/  TA          | Textabstand zur Linie (0,00...9,99)       
  

  SYMBOLE /DLG_FRAME/ Frame1;Frame2
  Extern * !Schicht;LinKode;LinArt;NiP;!LE;TKode; NKS;THoehe;TAbst
  SYMBOLE * Ret;F_Lang

  EINSYMBOLE "Topo_TextAusLinien.QPS" 
  
*..................Dialog anlegen
  Dialog = NEWOBJECT /DLG_WINDOW/
  Dialog.SetCaption("Texte fr Linienelementlnge und -neigung generieren")
  Dialog.SetHelp("TextausLin")
*..................Rahmen anlegen
  F_Lang = 135
  Frame1 = Dialog.CreateFrame(0;"Quell - Schicht")
  Frame1.SetSize(F_Lang)                              | Rahmenbreite festlegen
  Frame2 = Dialog.CreateFrame(0;"Einstellungen")  | Rahmen untereinander, sichtbar, mit Beschriftung 
  Frame2.SetSize(F_Lang)


*..................Feld fr Schichtauswahl + Bezeichnung
  Schicht = NewObject /DLG_LAYER/
  Schicht.SetLabel("Schicht:")
  Schicht.SetName(!Schicht)           | Vorbelegung mit Schichtname 
  Frame1.AddField(Schicht)
*..................Feld fr Liniencode + Bezeichnung
  Linienkode = NewObject /DLG_CODELINE/
  Linienkode.SetLabel("Linienkode:")
  Linienkode.SetCode(LinKode)
  Frame1.AddField(Linienkode)

*...................Feld fr Checkbox
  CheckS = NewObject /DLG_CHECKBOX/
  CheckS.SetBoxLabel("Raumstrecke verwenden")
  CheckS.SetFieldLabel(" ")
  CheckS.SetValue(LinArt)
  Frame2.AddField(CheckS)

*...................Feld fr Checkbox
  CheckN= NewObject /DLG_CHECKBOX/
  CheckN.SetBoxLabel("Neigungsbeschriftung erzeugen")
  CheckN.SetFieldLabel(" ")
  CheckN.SetValue(NIP)
  Frame2.AddField(CheckN)  

  Einheit= NewObject /DLG_TEXT/
  Einheit.SetSize(5)
  Einheit.SetLabel("Lngeneinheit:")
  Einheit.SetText(!LE)
  Frame2.AddField(Einheit)

 *...................Feld fr TextKode
  TxCode = NewObject /DLG_CODELINE/
     * Temporre Kodeauswahl TopoDaten
     * SetDart(Dart) undokumentiert
     * zulssige Werte fr Dart:
     *   0: Punkte
     *   1: Linien
     *   2: Gruppen
     *   4: Bume
     *   6: Symbole
     *   10: Achsen
     *   11: Bschungen
     *   13: Texte
     *   15: Flchen 
  TxCode.SetDart(13)
  TxCode.SetLabel("Textkode:")
  TxCode.SetCode(TKode)
  Frame2.AddField(TxCode)

 *...................Feld fr Nachkommastellen
  Komma = NewObject /DLG_COMBOBOX/
  Komma.SetSize(10)
  Komma.SetLabel("Nachkommastellen:") 
  Komma.AddText("0"; 0) 
  Komma.AddText("1"; 1) 
  Komma.AddText("2"; 2) 
  Komma.AddText("3"; 3) 
  Komma.SetValue(NKS)
  Frame2.AddField(Komma)

 *...................Feld fr Texthhe
  TH = NewObject /DLG_NUMBER/
  TH.SetSize(10;2)  
  TH.SetLabel("Texthhe:")
  TH.SetValue(THoehe)
  Frame2.AddField(TH)

 *...................Feld fr Textabstand
  TA = NewObject /DLG_NUMBER/
  TA.SetSize(10;3)  
  TA.SetLabel("Textabstand:")
  TA.SetValue(TAbst)
  Frame2.AddField(TA)
  
*....................Dialog ausfhren
  Ret = Dialog.DoDialog()
  
*....................Ergebnis
  WENN Ret DANN
    !Schicht = Schicht.GetName()
    LinKode  = Linienkode.GetCode()
    LinArt = CheckS.GetValue()
    NiP = CheckN.GetValue()
    !LE =  Einheit.GetText()
    TKode = TxCode.GetCode()
    NKS = Komma.GetValue()
    THoehe = TH.GetValue()
    TAbst = TA.GetValue()

    AUSSYMBOLE !Schicht;LinKode;LinArt;NiP;!LE;TKode; NKS;THoehe;TAbst

  WEITER
   
  ZURCK Ret

ENDE FUNKTION DIALOG



* -----------------------------------------------------------------------------
* @function LinienText
* @brief Elementlngen ermitteln und als Texte mittig der Elemente generieren
*        (Textschicht = Linienschicht)
* @param Linie: 
*
* @ingroup
* @author JH
* @endfunction
* -----------------------------------------------------------------------------
FUNKTION LinienText: {/TOPOLINE/} Linie
  EXTERN /TOPOSYS/ TopoSys                          | Symbol fr das System der Topografiedaten
  EXTERN /TOPOLAYER/ Schicht                        | Symbol fr die Schicht
  Extern * LinArt;NiP;!LE;TKode; NKS;THoehe;TAbst |LinKode;

  SYMBOLE /TOPOLINEELEMENT/ Element                  | Symbol fr das Linienelement
  SYMBOLE /TOPOTEXT/ LpText                          | Symbol fr den Topografietext
  SYMBOLE /COORDINATE/ LPCo; Ko1; Ko2; HPCo; HP2Co   | Symbole fr Hilfspunkte
  SYMBOLE /LINE/ HLinie1; HLinie2                    | Symbole fr Hilfslinien

  SYMBOLE * Eidx;LastStat;Typ;Vidx;Station;DiffStat;TStat;LPZ;VLPZ;Laenge;HLaenge;ElNeig;!Laenge
  SYMBOLE * !LinBez;LinNr;LWkl;!ElNeig  
  *** Schleife zum Auslesen der Linienelemente

  !LinBez = Linie.GetCodeDescription()           | liefert die Kodebezeichnung der Linie
  LinNr = Linie.GetNumber()                      | liefert die Liniennummer 


  Eidx = 1                                         | Index des 1. Linienelementes
  LastStat = 0                                     | Station des vorherigen Linienkleinpunktes 

  * Linienelementobjekt
  SOLANGE Element = Linie.GetElement(Eidx)         | liefert das Linienelement

    Typ = Element.GetType()                        | liefert den Elementtyp 
  
    *** Stationsermittlung fr Textposition
    Vidx = Linie.GetVertexIndex(Eidx)              | liefert den Index des Linienkleinpunktes 
    Station =  Linie.GetChainage(Vidx)             | liefert die Station des Linienkleinpunktes 
    DiffStat = Station - LastStat                  | ermittelt die Differenzstation zum vorherigen Element (= Elementlnge)
    TStat = LastStat + (DiffStat / 2)              | ermittelt die Textposition fr den zu erzeugenden Text (= mittig an Linienelement)
      
    * Hilfspunktobjekt "Elementpunkt"
    LPCo = NEWOBJECT /COORDINATE/                  | erzeugt ein neues Hilfsobjekt
    LPCo = Element.GetCoordinate()                 | liefert die Koordinate und Hhe des Elementpunktes    
    LPZ = LPCo.GetZ()                              | liefert die Punkthhe

    WENN Eidx == 1 DANN                            | 1. Element (Lnge = 0)
      VLPZ = LPZ                                   | Punkthhe
      Ko1 = LPCo                                   | Punktkoordinate
    SONST
    
      *** Textgenerierung "Elementlnge" mit gewhlten Nachkommastellen
    
      Laenge = Element.GetLength(LinArt)           | liefert die Elementlnge 
      WENN NKS = 0 DANN TEXTGEN !Laenge; #; Laenge
      WENN NKS = 1 DANN TEXTGEN !Laenge; #.#; Laenge
      WENN NKS = 2 DANN TEXTGEN !Laenge; #.##; Laenge
      WENN NKS = 3 DANN TEXTGEN !Laenge; #.###; Laenge 
      !Laenge = !Laenge + !LE                      | verbindet Text
    
      *** Textgenerierung "Elementneigung" mit gewhlten Nachkommastellen
    
      * Neigungsberechnung
      HLaenge = Element.GetLength(0)               | liefert die Horizontalstrecke
      ElNeig = (LPZ-VLPZ) / HLaenge*100            | Neigung in Prozent 
      WENN NiP = 2 DANN ElNeig = ElNeig / 10       | Neigung in Promille (zurzeit nicht implementiert)
 
      WENN NKS = 0 DANN TEXTGEN !ElNeig; #; ElNeig
      WENN NKS = 1 DANN TEXTGEN !ElNeig; #.#; ElNeig
      WENN NKS = 2 DANN TEXTGEN !ElNeig; #.##; ElNeig
      WENN NKS = 3 DANN TEXTGEN !ElNeig; #.###; ElNeig
    
      * Neigungseinheit: Prozent oder Promille (zurzeit nur Prozent)
      WENN NIP = 1 DANN !ElNeig = !Elneig + " %"   | verbindet Text
    
      *** Meldung   
    
      MELDE "Elementnr. #: Typ # Lnge #####.### Neigung ###.###"; EIdx; Typ; Laenge; ElNeig

      LastStat = Station  
      Ko2 = LPCo
    
      *** Ermittlung der Textposition und -drehung
    
      * Hilfsobjekte
      HLinie1 = NEWOBJECT /LINE/                     | erzeugt ein neues Hilfsobjekt 
      HLinie2 = NEWOBJECT /LINE/
    
      Hlinie1 = Linie.GetCrossLine(TStat)            | liefert senkrechte Linie auf dem Element an der Textstation
      Hlinie1.SetDistance(2)
      HLinie2 = HLinie1.GetCrossLine(0)              | liefert senkrechte Linie auf der Hilfslinie
      * Hlinie1.Draw(7; 2; 2)                        | visualisiert die ermittelte Lage und Richtung (Kontrolle)
      * Hlinie2.Draw(7; 2; 2)  
    
      LWkl = HLinie2.GetAngle()                      | liefert Textwinkel (= Tangential zur Linie)
      HPCo = HLinie1.GetLateralPoint(TAbst; 0)       | liefert Textposition fr "Elementlnge"
      HP2Co = HLinie1.GetLateralPoint(-TAbst; 0)     | liefert Textposition fr "Elementneigung"

      * Textobjekt "Elementlnge"
      LpText = Toposys.CreateText(Schicht; TKode)    | erzeugt einen neuen Text
      LpText.SetTextPosition(HPCo)                   | bernimmt Hilfspunkt als Textposition
      LpText.SetText(!Laenge)                        | setzt den Text
      LpText.SetAngle(LWkl)                          | setzt den Textwinkel
      LpText.SetFontHeight(THoehe)	               | setzt die Texthhe
      LpText.SetHorizontalOrient($TEXT_HOR_CENTERED) | setzt horizontale Textausrichtung: mittig
      LpText.SetVerticalOrient($TEXT_VER_TOP)	   | setzt vertikale Textausrichtung: oben
    
      * Textobjekt "Elementneigung"
      WENN NIP > 0 DANN
        LpText = Toposys.CreateText(Schicht; TKode)
        LpText.SetTextPosition(HP2Co)
        LpText.SetText(!ElNeig)
        LpText.SetAngle(LWkl)
        LpText.SetFontHeight(THoehe)	
        LpText.SetHorizontalOrient($TEXT_HOR_CENTERED)	
        LpText.SetVerticalOrient($TEXT_VER_BOTTOM)	
      WEITER
    
      Ko1 = Ko2
    
    WEITER
  
    EIdx = EIdx + 1 
    VLPZ = LPZ
  
  ENDE SOLANGE 
 
  *** Ende Schleife zum Auslesen der Linienelemente
 
ENDE FUNKTION

HILFE TEXT  Allgemein
    " Topografietexte fr Linienlngen und -neigungen generieren "
    " ---------------------------------------------------------- "
    " Mit diesem Script werden ausgewhlte Linien einer Schicht "
    " mit Elementlngen und Elementneigung beschriftet. Die Texte "
    " werden mittig an den Elementen oberhalb und unterhalb der "
    " Linie positioniert. "
    " Die Texte werden in die gleiche Schicht wie die Linien "
    " generiert."
    " "
    " "
ENDE HILFE
